/*
 * alternative Loesung zu Blatt 10
 */

#include <sys/types.h>	/* notendige ... */
#include <sys/stat.h>
#include <dirent.h>	/* ... Includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int hist_size[10];	/* Histogramm fuer erste Ziffer der Groesse */
int hist_atime[10];	/* Histogramm fuer erste Ziffer der Zugriffszeit */

/*
 * liefert die erste Ziffer der Integer i
 */
int first_digit(int i) {
    while (i >= 10)
	i /= 10;
    return i;
}

/*
 * geht rekursiv ueber alle Unterverzeichnisse von path und
 * berechnet dabei die Histogramme der ersten Ziffern von
 * Groesse und Zugriffszeit
 */
void recurse_subdirs(char *path) {
    DIR *dir;
    struct dirent *entry;

    /* Verzeichnis zum Lesen oeffnen */
    if (!(dir = opendir(path))) {
	char err[1024];
	sprintf(err, "opendir: %s", path);
	perror(err);
	return;
    }

    while ((entry = readdir(dir))) {
	char buf[1024];
	struct stat statbuf;

	/* Sonderbehandlung fuer . und ..
	 * => nichts tun und sofort zum naechsten Eintrag springen */
	if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, ".."))
	    continue;

	/* Pfadnamen konstruieren aus Pfad + Dateinamen */
	sprintf(buf, "%s/%s", path, entry->d_name);

	/* Info ueber Datei beschaffen (lstat geht auf symb. Links, statt auf deren Ziel) */
	if (lstat(buf, &statbuf) < 0) {
	    char err[1024];
	    sprintf(err, "stat: %s", buf);
	    perror(err);
	    continue;
	}

	/* Dateinamen ausgeben */
	puts(buf);

	/* alle Dateien ins Histogramm aufnehmen => an einer Stelle inkrementieren */
	hist_size[first_digit(statbuf.st_size)]++;
	hist_atime[first_digit(statbuf.st_atime)]++;

	/* wenn's sich um ein Verzeichnis handelt (jetzt ja nicht mehr . und ..!)
	 * dann bitte rekursiv behandeln */
	if (S_ISDIR(statbuf.st_mode))
	    recurse_subdirs(buf);
    }

    /* ... und zum Schluss die Verbindung zum Verzeichnis noch sauber beenden */
    closedir(dir);
}

int main(int argc, char **argv) {
    int i, n;

    /* genau ein Argument => andernfalls Usage-Meldung */
    if (argc != 2) {
	fprintf(stderr, "Usage: %s <dir>\n", argv[0]);
	exit(1);
    }

    /* beide Histogramme mit 0 initialisieren */
    for (i = 0; i < 10; i++)
	hist_size[i] = hist_atime[i] = 0;

    /* jetzt rek. in alle Unterverzeichnisse absteigen und Histogramm aktualisieren */
    recurse_subdirs(argv[1]);

    /* n ist die Summe der Haeufigkeiten in allen Klassen
     * => zur Berechnung der rel. Haeufigkeiten notwendig */
    for (i = 0, n = 0; i < 10; i++)
	n += hist_size[i];	/* n ist bei hist_atime identisch! */

    /* Histogramm fuer die erste Ziffer der Dateigroesse ausgeben */
    puts("Histogramm fr die Dateigre:");
    for (i = 0; i < 10; i++)
	/* sowohl absolute als auch rel. Haeufigkeit ausgeben */
	printf("Ziffer %d	%10d	%1.3f\n", i, hist_size[i], (float) hist_size[i] / (float) n);

    puts("");

    /* Histogramm fuer die erste Ziffer der Zugriffszeit ausgeben */
    puts("Histogramm fr die Zugriffs-Zeit:");
    for (i = 0; i < 10; i++)
	/* sowohl absolute als auch rel. Haeufigkeit ausgeben */
	printf("Ziffer %d	%10d	%1.3f\n", i, hist_atime[i], (float) hist_atime[i] / (float) n);

    exit(0);
}
